/*
 * LED_Driver.cpp
 *
 *  Created on: Nov 11, 2014
 *      Author:David Drake
 *
 Filled with simple functions that other libraries can call to make the LEDs flash
 */

#include <Energia.h>
#include "Blinds.h"

LED_Driver::LED_Driver()
{
	//recall which LEDs send which signals and set the state to 0

	_state=OFF;
	_lock=0;
	_flash_win=-1;

	pinMode(GOOD_LED, OUTPUT);
	pinMode(BAD_LED, OUTPUT);

	digitalWrite(GOOD_LED,HIGH);
	digitalWrite(BAD_LED,HIGH);

	Object_Admin::LED_Driver_Ref=this;
}

void LED_Driver::init(){
	//empty, no interupts or anything that needs initializing
}

void LED_Driver::flash_good(){
	if(!_lock){
		_state=FLASH_GOOD;
		digitalWrite(GOOD_LED,LOW);
		_make_window();
	}
}

void LED_Driver::flash_bad(){
	if(!_lock){
		_state=FLASH_BAD;
		digitalWrite(BAD_LED,LOW);
		_make_window();
	}
}
void LED_Driver::flash_all(){
	if(!_lock){
		_state=FLASH_ALL;
		digitalWrite(GOOD_LED,LOW);
		digitalWrite(BAD_LED,LOW);
		_make_window();
	}
}
void LED_Driver::solid_good(){
	if(!_lock){
		_state=SOLID_GOOD;
		digitalWrite(GOOD_LED,LOW);
	}

}
void LED_Driver::solid_bad(){
	if(!_lock){
		_state=SOLID_BAD;
		digitalWrite(BAD_LED,LOW);
	}
}
void LED_Driver::solid_all(){
	if(!_lock){
		_state=SOLID_ALL;
		digitalWrite(GOOD_LED,LOW);
		digitalWrite(BAD_LED,LOW);
	}
}
void LED_Driver::off(){
	//turn all off
	if(!_lock){
		_state=OFF;
		digitalWrite(GOOD_LED,HIGH);
		digitalWrite(BAD_LED,HIGH);
		_flash_win=-1;
	}
}

void LED_Driver::flash_off(){
	_state=OFF;
	digitalWrite(GOOD_LED,HIGH);
	digitalWrite(BAD_LED,HIGH);
	_flash_win=-1;
}

void LED_Driver::lock(){
	_lock=1;
}

void LED_Driver::unlock(){
	_lock=0;
}

void LED_Driver::_make_window(){
	//I basically run a polling system on the microcontroller.
	//Every loop, each object is given the opportunity to see if an event has happened for it.
	//Each loop, LED driver will see if a flash even has happened. If so, it will toggle the LED on/off
	_flash_win=(*Object_Admin::Schedule_Admin_Ref).get_ticks()+490;
}

void LED_Driver::check_flags(){
	int temp_ticks=(*Object_Admin::Schedule_Admin_Ref).get_ticks();
	if (_flash_win!=-1){
		if((_flash_win<TICKS&&temp_ticks>_flash_win)
				||(_flash_win>TICKS
				&&temp_ticks>(_flash_win-TICKS)
				&&temp_ticks<(_flash_win-490)))
		{
			if(_state==FLASH_GOOD){
				digitalWrite(GOOD_LED,HIGH);
				_state=_state*10;
			}else if (_state==FLASH_GOOD*10){
				digitalWrite(GOOD_LED,LOW);
				_state=FLASH_GOOD;
			}else if (_state==FLASH_BAD){
				digitalWrite(BAD_LED,HIGH);
				_state=FLASH_BAD*10;
			}else if (_state==FLASH_BAD*10){
				digitalWrite(BAD_LED,LOW);
				_state=FLASH_BAD;
			}else if (_state==FLASH_ALL){
				digitalWrite(GOOD_LED,HIGH);
				digitalWrite(BAD_LED,HIGH);
				_state=FLASH_ALL*10;
			}else if (_state==FLASH_ALL*10){
				digitalWrite(GOOD_LED,LOW);
				digitalWrite(BAD_LED,LOW);
				_state=FLASH_ALL;
			}
			_make_window();
		}
	}
}
