/*
 * Object_Admin.h
 *
 *  Created on: Nov 24, 2014
 *      Author: David Drake
 *
 *      This static class will hold references to all other admin and driver objects.
 *      objects will share common references through this object to reduce the amount of coding
 *      necessary to get them to talk to each other.  This object will wait until all objects are
 *      initialized, at which point it will initialize their opporation. (IE, once they will wait
 *      until this object is full, before they start passing messages to each other, this will
 *      avoid errors when the objects aren't initialized properly)
 */

#ifndef OBJECT_ADMIN_H_
#define OBJECT_ADMIN_H_

//use forward declarations for all other object classes
class Motor_Driver;
class Schedule_Admin;
class SD_Driver;
class XBEE_Driver;
class Client_Admin;
class LED_Driver;

#define LONG_BUF 150
#define SHORT_BUF 30

class Object_Admin
{
public:
	static void set_Motor_Driver(Motor_Driver *ref);
	static void set_Schedule_Admin(Schedule_Admin *ref);
	static void set_SD_Driver(SD_Driver *ref);
	static void set_XBEE_Driver(XBEE_Driver *ref);
	static void set_LED_Driver(XBEE_Driver *ref);

	//hold a static reference to all objects so they can communicate amongst each other
	 static Motor_Driver *Motor_Driver_Ref;
	 static Schedule_Admin *Schedule_Admin_Ref;
	 static SD_Driver *SD_Driver_Ref;
	 static XBEE_Driver *XBEE_Driver_Ref;
	 static Client_Admin *Client_Admin_Ref;
	 static LED_Driver *LED_Driver_Ref;
	 static unsigned char long_buffer[ LONG_BUF ];
	 static unsigned char short_buffer[ SHORT_BUF ];
};

#endif /* OBJECT_ADMIN_H_ */
