/*
 * Schedule_Admin.h
 *
 *  Created on: Nov 12, 2014
 *      Author: David Drake
 */

#ifndef SCHEDULE_ADMIN_H_
#define SCHEDULE_ADMIN_H_

#include <Energia.h>
#include <MspFlash.h>

//use a forward declarations as needed
class Object_Admin;
class Motor_Driver;
class XBEE_Driver;
class Client_Admin;


#define MINS_PER_WEEK 10080//num mins in a week
#define REPETITIVE_MEM ((unsigned char*)0x1040)//define the repetitive memory segment as segmentC
#define NON_REPETITIVE_MEM ((unsigned char*)0x1080)//define the individual memory segment as segmentB
#define WORD_REF 2//one word is 2 bytes


class Schedule_Admin
{
public:
	//initializer and init
	Schedule_Admin();
	void init();

	//public methods
	void stop_rtc_clock();
	void restart_rtc_clock(int new_time);
	void write_schedule(unsigned char *new_schd,int time_stamp);
	void stop_schedule();
	int get_ticks();
	void check_flags();
	void read_schedule(unsigned char *buf,int num_bytes);
	int get_local_time();
	void client_error_schd(unsigned char *buf);
	void client_normal_schd(unsigned char *buf);

	//internal timer has gone off, update ticker variable
	//since this is being called from isr it must be public. It should only be called from
	//isr so I am leaving the _ to indicate it should be treated as private
	void _update_rtc_clock();

private:

	//methods
	int _schedule_exists();
	void _find_next_event();//scan flash memory for next event
	unsigned int _rep_mem_ptr_to_timestamp(unsigned char* mem_cpy);//find next soonest time given a rep_mem word
	unsigned int _mem_ptr_to_timestamp(unsigned char mem_cpy[WORD_REF]);//find next soonest time given a regular mem word
	void _check_schedule();//the internal rtc clock has shifted to a new minute. See if an event needs to happen

	//variables
	unsigned int _next_event_time;//integer for next time (local timestamp) of event
	unsigned char _next_event_type;//represents action of next event (first two bits of the byte determin action)
	volatile unsigned int _rtc_clock;//keeps track of minutes since 12:00am Sunday. Restart after a week

};

#endif /* SCHEDULE_ADMIN_H_ */
