/*
 * XBEEDriver.cpp
 *
 *  Created on: Nov 11, 2014
 *      Author: David Drake
 */

#include <Energia.h>
#include "Blinds.h"
#include <spi.h>

void shift_bits(unsigned char *temp_buffer, int shift, int length){
	//takes an array and shifts the contents up by the number represented by shift.
	//WE DO NOT CHECK SPACE REQUIREMENTS (this is modifying the one single array which has
	//excess capacity for a 64 character password and 8 configuration bits
	int i;
	int max_bit=shift+length+1;
	//shift all bits and don't bother cleaning up the first bits
	for(i=max_bit-shift;i>-1;i--)
	{
		*(temp_buffer+i+shift)=*(temp_buffer+i);
	}
}

void prepare_check_sum(unsigned char *temp_buffer,unsigned char *CS_Location){
	//takes an array and shifts the contents up by the number represented by shift.
	//WE DO NOT CHECK SPACE REQUIREMENTS (this is modifying the one single array which has
	//excess capacity for a 64 character password and 8 configuration bits
	volatile unsigned int cs=0;
	unsigned char* temp_ptr=temp_buffer+3;

	//shift all bits and don't bother cleaning up the first bits
	while(temp_ptr!=CS_Location)
	{
		cs+=(*temp_ptr++);
	}
	cs=(cs&0xff);
	cs=0xff-cs;
	*CS_Location=cs;
}

void spi_tx (unsigned char *tx_buffer, int length){
	//given a message prepared in a given tx_buffer, send it to the xbee
	//length is the entire message length (including 7e and length bits and check sum
	for (int i=0;i<length;i++){
		(unsigned char)SPI.transfer((*(tx_buffer+i)&0xff));
		__delay_cycles(INTER_SPI_DELAY);
	}
}

int spi_rx (unsigned char *rx_buffer, byte message_type){
	int j=0;
	int length=0;

		for(int i=0;i<100;i++){
			__delay_cycles(INTER_SPI_DELAY);
			*(rx_buffer)=SPI.transfer(0xff);
			if(*rx_buffer==0x7e){
				j=0;//don't forget to reset j or else upon the first unsuccesful
				//when we loop back here, j will continue incrementing
				__delay_cycles(INTER_SPI_DELAY);
				*(rx_buffer+(++j))=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				*(rx_buffer+(++j))=SPI.transfer(0xff);
				length=(*(rx_buffer+1)&0xff)*16+*(rx_buffer+2)+2+j;

				for(j=j+1;j<length;j++){
					__delay_cycles(INTER_SPI_DELAY);
					*(rx_buffer+j)=SPI.transfer(0xff);
				}
				__delay_cycles(INTER_SPI_DELAY);

				switch(message_type)
				{
					case AT_PACKET:
						if(rx_buffer[3]==0x88&&rx_buffer[7]==0x00){
							return 1;
						}
						break;

					case IP_PACKET:
						//check that xbee has received a message to transmit packet
						if(rx_buffer[3]==0xb0){
							return 1;
						}
						break;

					case MODCON:
						//check that xbee has received a message to transmit packet
						if(rx_buffer[3]==0x8A&&rx_buffer[4]==0x02){
							return 1;
						}
						break;
				}
			}
		}
		return 0;
}

void prepare_AT_ID (unsigned char *tx_buf, int length)
{
	//compress the preparation of a WR command into this function

	/* first seven bits are header for xbee
	 * bit 0 is 7e
	 * bit 1 and 2 is length of message
	 * bit 3 is frame type 09 at command
	 * bit 4 is frame id 01
	 * bit 5 is at command 49 44 (id)
	 * put message
	 * finish with checksum
 */
	shift_bits(tx_buf,7,length);
	*tx_buf=0x7e;
	*(tx_buf+1)=00;
	*(tx_buf+2)=length+4;
	*(tx_buf+3)=0x09;//we use the queuing byte 0x09 since this is what xctu uses
	*(tx_buf+4)=0x01;
	*(tx_buf+5)=0x49;
	*(tx_buf+6)=0x44;
	//this is the shifted network name
	//I don't know why this is 4 and prepare PK is not 4
	prepare_check_sum(tx_buf, tx_buf+length+7);
}

void prepare_AT_PK (unsigned char *tx_buf, int length)
{
	//compress the preparation of a WR command into this function

	/* first seven bits are header for xbee
	 * bit 0 is 7e
	 * bit 1 and 2 is length of message
	 * bit 3 is frame type 09 at command
	 * bit 4 is frame id 01
	 * bit 5 is at command 50 4B (pk)
	 * put message
	 * finish with checksum
 */
	shift_bits(tx_buf,7,length);
	*tx_buf=0x7e;
	*(tx_buf+1)=00;
	*(tx_buf+2)=length+4;
	*(tx_buf+3)=0x09;//we use the queuing byte 0x09 since this is what xctu uses
	*(tx_buf+4)=0x01;
	*(tx_buf+5)=0x50;
	*(tx_buf+6)=0x4b;
	//this is the shifted network name
	prepare_check_sum(tx_buf, tx_buf+length+7);
}

void prepare_IP (unsigned char *tx_buf, int length, byte ip_address[4], byte port[2]){
	int i=0;

	shift_bits(tx_buf,15,length);
	*(tx_buf+i++)=0x7e;
	*(tx_buf+i++)=0x00;
	*(tx_buf+i++)=12+length;
	*(tx_buf+i++)=0x20;//0x20=wifi frame
	*(tx_buf+i++)=0x00;//frame id
	*(tx_buf+i++)=ip_address[0];//following four bits represent the ip
	*(tx_buf+i++)=ip_address[1];
	*(tx_buf+i++)=ip_address[2];
	*(tx_buf+i++)=ip_address[3];
	*(tx_buf+i++)=port[0];//this and next byte are the port
	*(tx_buf+i++)=port[1];
	*(tx_buf+i++)=0x00;//this and next are source port (must be 0 for new connection)
	*(tx_buf+i++)=0x00;
	*(tx_buf+i++)=0x01;//TCP
	*(tx_buf+i++)=0x00;//leave socket open (let it time itself closed)
	prepare_check_sum(tx_buf, tx_buf+length+15);
}

//include miso and mosi as reminders (do not use)
XBEE_Driver::XBEE_Driver()
{
	//by default we are not internet enabled (must get a good password and network id from sd card)
	_internet_enabled=0;
	_network_enabled=0;
	_pwd_enabled=0;

	//hardcode a destination address. In the future, we should use DNS to lookup the address
	_ip_address[0]=0x82;
	_ip_address[1]=0x71;
	_ip_address[2]=0x36;
	_ip_address[3]=0x9e;

	//hardcode a destination port
	_port[0]=0x27;
	_port[1]=0x09;

	//server flag indicates that we are waiting for a response from the server
	//set to 0, since we haven't sent a message yet
	_gen_counter=0;
	//there are various stages that happen when the xbee is turned on.  This variable keeps track of those
	//stages
	_xb_flag=0;

	//turn off object sharing variables since we are not interacting with other objects yet
	_tx_buf=0;
	_rx_buf=0;
	_fctn=0;
	_msg_length=0;

	//prepare sleep
	pinMode(XB_SLEEP,OUTPUT);
	digitalWrite(XB_SLEEP,SLEEP);

	//turn off SPI select
	pinMode((int)XB_SPI_SEL,OUTPUT);
	digitalWrite((int)XB_SPI_SEL,SPI_OFF);

	//ensure that mosi, miso and sclk are configured properly. NOTE we do not
	//remember these variables to save space and because they do not change
	pinMode(SCLK,OUTPUT);
	pinMode(MISO,INPUT);
	pinMode(MOSI,OUTPUT);

	Object_Admin::XBEE_Driver_Ref=this;

	SPI.setBitOrder(MSBFIRST);
	SPI.begin();//be careful, don't know if there will be issues when initializing spi a second time
}

void XBEE_Driver::temp(unsigned char *tx_buf,unsigned char *rx_buf, int length){
}

void XBEE_Driver::send_message(unsigned char *tx_buf,unsigned char *rx_buf, byte length,void (*fctn)(unsigned char *resp, int length)){
	//send a message as long as we have an internet connection and are not waiting for a response
		if(_internet_enabled&&!_xb_flag){
			//save object interaction variables for future reference
			_tx_buf=tx_buf;
			_rx_buf=rx_buf;
			_fctn=fctn;
			_msg_length=length;

			//wake up xbee, prepare a message and send it
			_wake();
			_xb_flag=WAIT_FOR_CONNECTION;
			_gen_counter=TURN_ON_WAIT_PERIOD;
		}
}

void XBEE_Driver::init()
{
}

void XBEE_Driver::set_offline(){
	//turn off XBEE
	_internet_enabled= 0;
}

unsigned char XBEE_Driver::get_activated(){
	//return if xbee is on or not
	return _internet_enabled;
}

void XBEE_Driver::set_network(unsigned char *tx_buf,unsigned char *rx_buf, int length)
{
//any strings are held in the one and only com_buffer. We don't have memory capacity to declare a second
//string. Instead, modify this buffer to become an acceptable message

	//due to limited ram, I have to reuse the same buffer. Once I send the network ID, I must
	//reuse the buffer to send the WR and AC messages. In the process I lose the network ID.
	//This is why I have structured my functions so that _send_configuration will resend if
	//the first transmission is unsuccesful, rather than resending all three messages. This
	//may be a weakness in the future.

	_gen_counter=0;

	//wake xbee
	_wake();
	__delay_cycles(INTER_MESSAGE_DELAY);
	//send password
	prepare_AT_ID(tx_buf,length);
	while (_gen_counter<50){
		__delay_cycles(INTER_MESSAGE_DELAY);
		if(_send_configuration(tx_buf,rx_buf, length+8)){
			//write the changes in

			if(_send_WR()){
				//apply changes
				if(_send_AC()){
					_network_enabled=1;
					_check_internet_enabled();
					break;
				}
			}
		}
		_gen_counter++;
		_sleep();
		__delay_cycles(INTER_MESSAGE_DELAY);
		_wake();
	}

	//put Xbee back to sleep
	_sleep();
	_gen_counter=0;

}

void XBEE_Driver::set_password(unsigned char *tx_buf,unsigned char *rx_buf,int length)
{
	_gen_counter=0;

	//wake xbee
	_wake();
	__delay_cycles(INTER_MESSAGE_DELAY);
	//send password
	prepare_AT_PK(tx_buf,length);
	while (_gen_counter<50){

		if(_send_configuration(tx_buf,rx_buf, length+8)){
			//write the changes in
			if(_send_WR()){
				//apply changes
				if(_send_AC()){
					_pwd_enabled=1;
					_check_internet_enabled();
					break;
				}
			}
		}
		_gen_counter++;
		_sleep();
		__delay_cycles(INTER_MESSAGE_DELAY);
		_wake();
	}

	//put Xbee back to sleep
	_sleep();
	_gen_counter=0;

}

int XBEE_Driver::_send_IP(unsigned char *tx_buffer, unsigned char *rx_buffer, int length){
	//send the message once, look for success
	//TODO must put in a mechanism to wait until
	spi_tx(tx_buffer,length);
	//give time for XBEE to prepare a response
	__delay_cycles(INTER_MESSAGE_DELAY);
	if(spi_rx(rx_buffer,IP_PACKET)){
		return 1;
	}
	return 0;
}

int XBEE_Driver::_send_configuration(unsigned char *tx_buffer, unsigned char *rx_buffer, int length){
	int trials=0;
	int success=0;

	//try five times to configure xbee
	while (trials<60&&success==0){
		spi_tx(tx_buffer,length);
		//give time for XBEE to prepare a response
		__delay_cycles(INTER_MESSAGE_DELAY);

		if(spi_rx(rx_buffer,AT_PACKET)){
				success=1;
		}
		else
		{
			trials++;
			__delay_cycles(INTER_MESSAGE_DELAY);
		}
	}

	return success;
}

void XBEE_Driver::_sleep(){
	//reset XBee, This seems to force it to listen to sleep requests it might not otherwise listen to
	//we may need to get the xbee to sleep at various times when

	int trials=0;
	int success=0;
	_sr=0;

	//try enough times to configure xbee
	while (trials<50&&success==0)
	{
		trials++;

		__delay_cycles(INTER_MESSAGE_DELAY);
		//transmit the reset message
		SPI.transfer(0x7e);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x00);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x04);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x08);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x01);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x46);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x52);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x5e);
		__delay_cycles(INTER_MESSAGE_DELAY);

		for(int i=0;i<100;i++)
		{
			_sr=SPI.transfer(0xff);
			__delay_cycles(INTER_SPI_DELAY);

			if(_sr==0x7e)
			{
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x88)
				{
					success=1;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x00&&success==1)
				{
					success=1;
					break;
				}
				else
				{
					success=0;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
			}
		}
	}

	//regardless of success, assert pins that will put xbee to sleep
	digitalWrite(XB_SPI_SEL,SPI_OFF);
	__delay_cycles(1000);
	digitalWrite(XB_SLEEP,SLEEP);
	__delay_cycles(10000);
}

int XBEE_Driver::_send_AC(){
	int trials=0;
	int success=0;
	_sr=0;

	//try enough times to configure xbee
	while (trials<50&&success==0)
	{
		trials++;
		__delay_cycles(INTER_MESSAGE_DELAY);
		//transmit the reset message
		SPI.transfer(0x7e);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x00);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x04);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x09);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x01);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x41);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x43);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x71);

		__delay_cycles(INTER_MESSAGE_DELAY);

		for(int i=0;i<100;i++)
		{
			_sr=SPI.transfer(0xff);
			__delay_cycles(INTER_SPI_DELAY);

			if(_sr==0x7e)
			{
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x88)
				{
					success=1;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x00&&success==1)
				{
					success=1;
					break;
				}
				else
				{
					success=0;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
			}
		}
	}
	return success;
}

int XBEE_Driver::_send_WR(){
	//reset XBee, This seems to force it to listen to sleep requests it might not otherwise listen to
	//we may need to get the xbee to sleep at various times when
	int trials=0;
	int success=0;
	_sr=0;

	//try enough times to configure xbee
	while (trials<50&&success==0)
	{
		trials++;

		__delay_cycles(INTER_MESSAGE_DELAY);
		//transmit the reset message
		SPI.transfer(0x7e);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x00);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x04);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x09);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x01);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x57);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x52);
		__delay_cycles(INTER_SPI_DELAY);
		SPI.transfer(0x4c);
		__delay_cycles(INTER_MESSAGE_DELAY);

		for(int i=0;i<100;i++)
		{
			_sr=SPI.transfer(0xff);
			__delay_cycles(INTER_SPI_DELAY);

			if(_sr==0x7e)
			{
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x88)
				{
					success=1;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
				if(_sr==0x00&&success==1)
				{
					success=1;
					break;
				}
				else
				{
					success=0;
				}

				_sr=SPI.transfer(0xff);
				__delay_cycles(INTER_SPI_DELAY);
			}
		}
	}
	return success;
}

void XBEE_Driver::_wake(){
	digitalWrite(XB_SLEEP,AWAKE);
	__delay_cycles(INTER_SPI_DELAY);
	digitalWrite(XB_SPI_SEL,SPI_ON);
	__delay_cycles(INTER_SPI_DELAY);
}

void XBEE_Driver::_check_internet_enabled(){
	//software restart will force xbee to connect, since I now employ software restarts whenever I put xbee to sleep,
	//we will only set our connected flag in this function
	if (_network_enabled&&_pwd_enabled){
			_internet_enabled=1;
	}
}

void XBEE_Driver::check_flags(){
	/* in Xbee_driver, we will delay 1000 cycles, this will cause _gen_counter to be
	 * waiting for a more predictable delay, there will be at least 1000 cycles wasted,
	 * adding a new check flags routine will not effect this too much because each check
	 * flags routine will be only a few cycles long, and not too long with respect to the wasted
	 * 1000 cycles
	 */
	__delay_cycles(1000);
	if(_xb_flag==WAIT_FOR_CONNECTION && _gen_counter>0){
		if(spi_rx (_rx_buf,MODCON)){
			prepare_IP(_tx_buf,_msg_length,_ip_address,_port);
			//allow for an extra long connection time
			__delay_cycles(INTER_MESSAGE_DELAY*2);
			__delay_cycles(INTER_MESSAGE_DELAY*2);
			__delay_cycles(INTER_MESSAGE_DELAY*2);
			__delay_cycles(INTER_MESSAGE_DELAY*2);
			spi_tx(_tx_buf,_msg_length+16);
			_gen_counter=WAIT_PERIOD;
			_xb_flag=WAIT_FOR_SRV_RESPONSE1;
		}else{
			//decrement time_delay to avoid doing this action forever
			_gen_counter--;
		}
	}

	else if(_xb_flag==WAIT_FOR_SRV_RESPONSE1 && _gen_counter>0){
		if(spi_rx(_rx_buf, IP_PACKET)){
			_gen_counter=WAIT_PERIOD;
			_xb_flag=WAIT_FOR_SRV_RESPONSE2;
		}
		else{
			_gen_counter--;
		}
	}

	else if(_xb_flag==WAIT_FOR_SRV_RESPONSE2 && _gen_counter>0){
		//try to receive message from server
		if(spi_rx (_rx_buf,IP_PACKET)){
			//put Xbee to slee, call the response function giving it the server response
			_sleep();
			_xb_flag=IDLE;



			//return a pointer to the message (skip overhead) and indicate message length
			_fctn(_rx_buf+14, *(_rx_buf+2)-11);

		}else{
			//decrement time_delay to avoid doing this forever
			_gen_counter--;
		}
	}else if(_xb_flag&&_gen_counter==0){
		//we have run out of time delay. turn off xbee and reset everything
		_sleep();
		_xb_flag=IDLE;



		//return a pointer to the message (skip overhead) and pass a -1 for length indicating an error
		_fctn(_rx_buf, -1);

	}
}
