/*
 * XBEEDriver.h
 *
 *  Created on: Nov 11, 2014
 *      Author: Room 12
 */

#ifndef XBEE_DRIVER_H_
#define XBEE_DRIVER_H_

#include <Energia.h>

#define SLEEP 1
#define AWAKE 0
#define SPI_ON 0
#define SPI_OFF 1
#define INTER_SPI_DELAY 2000 //in between bytes sent through SPI
#define INTER_MESSAGE_DELAY 15000//in between packets sent through SPI
#define WAIT_PERIOD 500//this is the wait period that the xbee_driver will wait for the xbee to
//send a response, or to connect to a network
#define TURN_ON_WAIT_PERIOD 500//give more time when turning on xbee
//xbee flag state variables
#define IDLE 0
#define WAIT_FOR_CONNECTION 1
#define WAIT_FOR_SRV_RESPONSE1 2
#define WAIT_FOR_SRV_RESPONSE2 3

//SPI Receive Types
#define AT_PACKET 1
#define IP_PACKET 2
#define MODCON 3

//use forward declaration so that xbee can reference the Schedule_Admin object
class Schedule_Admin;


class XBEE_Driver
{
public:

	XBEE_Driver();
	void init();
	void check_server();
	void set_offline();
	unsigned char get_activated();
	void set_network(unsigned char *tx_buf,unsigned char *rx_buf, int length);
	void set_password(unsigned char *tx_buf,unsigned char *rx_buf,int length);
	void check_flags();
	void temp(unsigned char *tx_buf,unsigned char *rx_buf, int length );
	void send_message(unsigned char *tx_buf,unsigned char *rx_buf, byte length,void (*fctn)(unsigned char *resp, int length));

private:

	//xbee pin variables
	byte _sleep_rx;
	byte _spi_sel;

	//xbee state variables
	byte _internet_enabled;
	byte _network_enabled;
	byte _pwd_enabled;
	unsigned long int _gen_counter;
	byte _xb_flag;

	//addressing variables
	byte _ip_address[4];
	byte _port[2];

	//xbee object sharing/response variables
	unsigned char *_tx_buf;
	unsigned char *_rx_buf;
	byte _msg_length;
	void(*_fctn)(unsigned char *resp, int length);

	//variable to read for sleep
	unsigned char _sr;

	//methods
	void _parse_packet();
	int _send_configuration(unsigned char *tx_buffer, unsigned char *rx_buffer, int length);
	int _send_IP(unsigned char *tx_buffer, unsigned char *rx_buffer, int length);
	int _receive_config_response(char *temp_buffer);
	void _check_internet_enabled();
	void _wake();
	void _sleep();
	int _send_WR();
		int _send_AC();


};

#endif /* XBEE_DRIVER_H_ */
