import java.awt.*;
import java.awt.image.BufferedImage;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.text.NumberFormatter;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.lang.reflect.*;
import java.util.Date;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.TimeZone;


public class wizard_swing extends JFrame {

	//keep track of gui
	int width = 800;
	int height = 600;
	int max_page = 7;
	int current_page=0;
	
	//path to image files
	String icon="/1.png";
	String placeholder="placeholder.png";
	String blinds_length="blinds_length.png";
	String blinds_diameter="blinds_diameter.png";
	String blinds_orient_out1="blinds_b.png";
	String blinds_orient_out2="blinds_d.png";
	String blinds_orient_in1="blinds_a.png";
	String blinds_orient_in2="blinds_c.png";
	String internet="router.png";
	String small_internet="router_small.png";
	String no_small_internet="no_router_small.png";
	
	//global interactive gui labels to be processed later
	JFormattedTextField field1;
	JPasswordField field2;
	JComboBox list;
	JRadioButton but1;
	JRadioButton but2;
	JRadioButton but3;
	JRadioButton but4;
	
	JComboBox day_box;
	JTextField hour;
	JTextField min;
	JComboBox am_pm_box;
	JComboBox up_down_box;
	JPanel day_panel[]=new JPanel[7];
	JCheckBox over_write_box;
	
	//output variables
	int b_length=0;
	int b_rad=0;
	int up_down=0;
	int img_selected=0;
	int connected=1;//assume by default that these blinds will be connected
	String network="";
	String pwd="";
	int local_timestamp;
	int srv_schd_overwritten=0;//by default we do not overwrite the schedule on the server
	//if the user enters a schedule, this server schedule will be overwritten
	
	//reserve schedule space that matches the available space in microcontroller for non-repetitive memory
	//dealing with sorting the events is excessively complicated at this stage
	int sched_array[]=new int[32];
	
	//create a format for numbers only text field
	NumberFormat format = NumberFormat.getNumberInstance();
	
	public class sched_full extends Exception {
	    public sched_full(String msg) {
	        super(msg);
	    }
	}

	public wizard_swing() {

		//get info on screen size
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();

		//create the window
		setTitle("Shade Soleil Configuration Wizard");
		setSize(width, height);
		setLocation((int) dim.getWidth() / 2 - width / 2, (int) dim.getHeight()
				/ 2 - height / 2);
		setResizable(false);

		//set some default actions
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setVisible(true);
				
		screen0("");
		
	}
	
	//input the path to an image file and this will create a jlabel with that image
	public JLabel Image_2_JLabel(String path){
	
		//import icon
		//File file = new File(path);
		
		try {
			//BufferedImage image = ImageIO.read(file);
			ImageIcon img=new ImageIcon(getClass().getResource(path));
			//JLabel label = new JLabel(new ImageIcon(image));
			JLabel label = new JLabel(img);
			add(label);
			return label;
		} catch (Exception e) {
			JLabel label = new JLabel("Could not find: "+path,JLabel.CENTER);
			return label;
		}
	}
	
	public JPanel prepare_template()
	{
		//clear screen
		getContentPane().removeAll();
		
		//let's set this as border layout so we can add a template footnote
		JPanel master_panel=new JPanel();
		master_panel.setLayout(new BorderLayout());
		
		//prepare a panel in the center of the gui to be filled with content
		JPanel return_panel=new JPanel();
		master_panel.add(return_panel,BorderLayout.CENTER);
		
		//initialize a variable width column based panel
		//initialize the constraints that are relevant to all buttons here
		JPanel bottom_panel=new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.fill=GridBagConstraints.BOTH;
		//add this bottom panel now. The following conditional will fill the bottom
		//panel
		master_panel.add(bottom_panel,BorderLayout.SOUTH);
		
		if (current_page==0)
		{
			//if this is the first page, do not include a back button
			JLabel blank1=new JLabel(" ");
			
			c.weightx=90;
			bottom_panel.add(blank1,c);
			
			JButton btn=new JButton("Start");
			btn.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					next();
				}
			});
			c.weightx=10;
			bottom_panel.add(btn,c);
			this.getRootPane().setDefaultButton(btn);
		}
		else if (current_page==max_page)
		{
			//if this is the last page, do not include a next button, label it finish
			JLabel blank1=new JLabel(current_page+" of "+max_page,SwingConstants.LEFT);
			blank1.setHorizontalAlignment(SwingConstants.LEFT);
			c.weightx=80;
			bottom_panel.add(blank1,c);
			
			JButton btn=new JButton("Back");
			btn.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					previous();
				}
			});
			c.weightx=10;
			bottom_panel.add(btn,c);
			
			JButton btn2=new JButton("Finish");
			btn2.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					finish();
				}
			});
			c.weightx=10;
			bottom_panel.add(btn2,c);
			this.getRootPane().setDefaultButton(btn2);
		}
		else
		{
			//middle pages have a next and back button
			JLabel blank1=new JLabel(current_page+" of "+max_page,SwingConstants.LEFT);
			blank1.setHorizontalAlignment(SwingConstants.LEFT);
			c.weightx=80;
			bottom_panel.add(blank1,c);
			
			JButton btn=new JButton("Back");
			btn.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					previous();
				}
			});
			c.weightx=10;
			bottom_panel.add(btn,c);
			
			JButton btn2=new JButton("Next");
			btn2.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					next();
				}
			});
			c.weightx=10;
			bottom_panel.add(btn2,c);
			this.getRootPane().setDefaultButton(btn2);
		}
		
		add(master_panel);
		setVisible(true);
		return return_panel;
	}
	
	public void next()
	{
		//called by a generic button. 
		//check page input
		String msg=check_save_input();
		if(msg=="")
		{
			//check for page exceptions
			if(connected==0&&current_page==4){
				current_page=5;
			}
			//update current page
			current_page++;
			
			//use reflection to call the next page
			try
			{
			Method method = this.getClass().getMethod("screen" + current_page, new Class[]{String.class});
			method.invoke(this, "");
			}
			catch (Exception e)
			{
				System.err.println(e.getMessage());
				System.exit(0);
			}
		}else{
			try
			{
			Method method = this.getClass().getMethod("screen" + current_page, new Class[]{String.class});
			method.invoke(this, msg);
			}
			catch (Exception e)
			{
				System.err.println(e.getMessage());
				System.exit(0);
			}
		}
	}
	
	public void finish()
	{
		//check page input
		
	
		System.exit(0);
		
	}
	
	public void previous()
	{
		//called by a generic button. 
		String msg=check_save_input();
		//if input is bad, it will not be saved, and a message will be returned
		//HOWEVER, since we're going back, don't show the message, just move
		//to the previous page
		
		//update current page
		current_page--;
		
		if(current_page==5&&connected==0){
			current_page--;
		}
		//use reflection to call the next page
		try
		{
			Method method = this.getClass().getMethod("screen" + current_page, new Class[]{String.class});
			method.invoke(this, "");
		}
		catch (Exception e)
		{
			System.exit(0);
		}
		
	}
	
	public String check_save_input()
	{
		double temp;
		String ans="";
		String temp_str;
		
		switch (current_page){
		case 1:
			try{
				temp=Double.parseDouble(field1.getText());
				
				if(list.getSelectedItem().toString().equals("mm"))
				{
					temp=temp;
				}else if (list.getSelectedItem().toString().equals("cm")){
					temp=temp*10;
				}else if(list.getSelectedItem().toString().equals("m")){
					temp=temp*100;
				}else if(list.getSelectedItem().toString().equals("inches")){
					temp=temp*24;
				}else if(list.getSelectedItem().toString().equals("feet")){
					temp=temp*304.8;
				}else{
					ans= "Unknown error relating to units";
				}
				
				if((int)Math.floor(temp)>999999)
				{
					ans="This value is too large";
				}else if ((int)Math.floor(temp)<1)
				{
					ans="This value is too small";
				}
				
				else{
					b_length=(int)Math.floor(temp);
				}
				
			} catch (NumberFormatException e){
				ans= "Please enter a valid number";
			}
			
			break;
		case 2:
			try{
				temp=Double.parseDouble(field1.getText());
				
				if(list.getSelectedItem().toString().equals("mm"))
				{
					temp=temp;
				}else if (list.getSelectedItem().toString().equals("cm")){
					temp=temp*10;
				}else if(list.getSelectedItem().toString().equals("m")){
					temp=temp*100;
				}else if(list.getSelectedItem().toString().equals("inches")){
					temp=temp*24;
				}else if(list.getSelectedItem().toString().equals("feet")){
					temp=temp*304.8;
				}else{
					ans= "Unknown error relating to units";
				}
				
				
				if((int)Math.floor(temp)>150)
				{
					ans=Integer.toString((int)Math.floor(temp))+" is too large value is too large";
				}else if ((int)Math.floor(temp)<1)
				{
					ans="This value is too small";
				}else{
					b_rad=(int)Math.floor(temp);
				}
				
			} catch (NumberFormatException e){
				ans= "Please enter a valid number";
			}
			
			break;
		case 3:
			ans="";
			if (but1.isSelected())
			{
				up_down=1;
				img_selected=1;
			} else if (but2.isSelected())
			{
				up_down=1;
				img_selected=2;
			}else if (but3.isSelected())
			{
				up_down=-1;
				img_selected=3;
			}else if (but4.isSelected())
			{
				up_down=-1;
				img_selected=4;
			}else
			{
				ans="You must select one of the images before proceding";
			}
			
			break;
			
		case 4:
			if(but1.isSelected()){
				//offline mode, skip network and password
				connected=0;
			}else if (but2.isSelected()){
				connected=1;
				//take no action, just move along
			}else{
				//one of the two must be selected
				ans="Please make a selection";
			}
			break;
		case 5:
			ans="";
			temp_str=field1.getText();
			if(temp_str.length()>32)
			{
				ans="The network name entered is too long";
			}
			else
			{
				network=temp_str;
			}
			temp_str=new String(field2.getPassword());
			if(temp_str.length()>64)
			{
				ans="The password entered is too long";
			}
			else
			{
				pwd=temp_str;
				//if its all good, write the sd card now in case the user ejects
				//without clicking finish
				
			}
			
			break;
			
		case 6:
			//nothing to be checked here. All input is through fields and is already controlled
			//write to the SD card
			write_sd();	
		}
		return ans;
	}
	
	public void write_sd()
	{
		//write values to SD card
		//NOTE: we will convert time to UTC in this function
		String temp;
		int i;
		long temp2;
		int length;
		//save to config.txt
		BufferedWriter out=null;
		BufferedOutputStream b_out=null;
		int sched_array2[]=new int[32];
		
		TimeZone z;
		int utc_factor;
		
		try
		{
			FileWriter fstream=new FileWriter("config.txt",false);
			out=new BufferedWriter(fstream);
			
			
			temp=Integer.toString(b_length);
			for (i=temp.length();i<6;i++)
			{
				temp="\0"+temp;
			}
			out.write(temp);
			
			temp=Integer.toString(b_rad);
			for (i=temp.length();i<6;i++)
			{
				temp="\0"+temp;
			}
			out.write(temp);
			
			temp=Integer.toString(up_down);
			for (i=temp.length();i<6;i++)
			{
				temp="\0"+temp;
			}
			out.write(temp);
			
			temp=network;
			for (i=temp.length();i<64;i++)
			{
				temp="\0"+temp;
			}
			out.write(temp);
			
			temp=pwd;
			for (i=temp.length();i<64;i++)
			{
				temp="\0"+temp;
			}
			out.write(temp);
			//we're done with characters
			out.close();
			fstream.close();
			
			//allow appending
			b_out=new BufferedOutputStream(new FileOutputStream("config.txt",true));
			
			//start work on time
			Calendar cal=Calendar.getInstance();

			//prepare a local timestamp (minutes since midnight sunday)
			local_timestamp=0;
			
			//determine factor necessary to convert system time back to UTC
			z=cal.getTimeZone();
			utc_factor=z.getRawOffset();
			if(z.inDaylightTime(new Date())){
				utc_factor=utc_factor+z.getDSTSavings();
			}
			utc_factor=-(utc_factor/1000/60);
			
			//adjust to UTC time
			cal.add(Calendar.MINUTE,utc_factor);
			
			switch (cal.get(Calendar.DAY_OF_WEEK)){
			case Calendar.SUNDAY:
				local_timestamp=0;
				break;
			case Calendar.MONDAY:
				local_timestamp=1440;
				break;
			case Calendar.TUESDAY:
				local_timestamp=1440*2;
				break;
			case Calendar.WEDNESDAY:
				local_timestamp=1440*3;
				break;
			case Calendar.THURSDAY:
				local_timestamp=1440*4;
				break;
			case Calendar.FRIDAY:
				local_timestamp=1440*5;
				break;
			case Calendar.SATURDAY:
				local_timestamp=1440*6;
				break;
			}
			
			local_timestamp+=cal.get(Calendar.HOUR_OF_DAY)*60;
			local_timestamp+=cal.get(Calendar.MINUTE)+2;
			
			write_word(b_out,local_timestamp);
			
			//system currenttiemillis is fixed in time, unaffected by timezones
			long temp3;
			if (srv_schd_overwritten==0){
				temp3=0;
			}else{
				temp3=System.currentTimeMillis()/1000L;
			}
			
			write_long(b_out,temp3);			
			
			write_word(b_out,18432);//this is the value to check server once every 5 minutes
			//this value doesn't matter because the msp will overwrite it with it's own scheduling tool
			
			for (i=0;i<27;i++)
			{
				//write blanks (no morenon repetitive memory is being used
				write_word(b_out,0);
			}
			
			int temp10;
			int temp11;
			
						
			for(i=0;i<32;i++){
				/*I MADE LIFE HARD HERE
				 * SO sched_array doesn't have a 2d array with event time and event type like it's stored
				 * Rather, I copy it into its proper format as I go along. The problem with this is that 
				 * everything works find EXCEPT that the final value does not consider UTC time.
				 * 
				 * SO I am going to fix that here, not in the previous code because I don't want to 
				 * mess with error control and flow of the wizard
				 */
				
				//correct to utc
				if((sched_array[i])==0){
					//if there is no scheduled item, skip ahead
					temp10=0;
				}
				else if((sched_array[i]&0x3Fff)+utc_factor<0){
					temp10=sched_array[i]+10080+utc_factor;
				}
				else if ((sched_array[i]&0x3Fff)+utc_factor>10080){
					temp10=sched_array[i]-10080+utc_factor;
				}
				else
				{
					temp10=sched_array[i]+utc_factor;
				}
				
				if(temp10!=0){
					for(int j=0;j<31;j++){
						
						if((sched_array2[j]&0x3fff)>(temp10&0x3fff)||sched_array2[j]==0){
							temp11=sched_array2[j];
							for(j=j+1;j<31;j++){
								sched_array2[j-1]=temp10;
								temp10=temp11;
								temp11=sched_array2[j];
							}
							sched_array2[31]=temp11;
						}
					}
				}
			}
			
			//write the corrected times
			for(i=0;i<32;i++){
				write_word(b_out,(sched_array2[i]));
			}

			
			b_out.close();
		}catch (IOException e)
		{
			System.exit(0);
		}
	}
	
	public void write_word(BufferedOutputStream out, int to_write){
		try{
			int i;
			int temp;
			
			for(i=1;i>=0;i--)
			{
				temp=to_write;
				out.write((char)((temp>>8*i)&255));
			}
		}catch (IOException e){
			System.exit(0);
		}
	}
	
	public void write_long(BufferedOutputStream out, long to_write){
		try{
			int i;
			long temp;
			char temp2[]=new char[1];
			for(i=7;i>=0;i--)
			{
				
				temp=to_write;
				temp=(temp>>>8*i)&255;
				temp2[0]=(char)temp;
				out.write((int)temp);
			}
		}catch (IOException e){
			System.exit(0);
		}
	}
	
	public void screen0(String error_message){
		//don't use this string, add it to make this consistent with other pages
		
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		//add content for the welcome screen
		content.setLayout(new BoxLayout(content, BoxLayout.PAGE_AXIS));

		JLabel img=Image_2_JLabel(icon);
		img.setAlignmentX(Component.CENTER_ALIGNMENT);
		content.add(img);
		
		JLabel txt= new JLabel("Wel"
				+ "come",JLabel.CENTER);
		txt.setFont (txt.getFont ().deriveFont (30f));
		txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		content.add(txt);
		
		JLabel txt2= new JLabel("This wizard will guide you through the steps in order to configure your new blinds. Just click \"Start\"!",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		content.add(txt2);
		
		JLabel txt3= new JLabel(" ",JLabel.CENTER);
		txt3.setAlignmentX(Component.CENTER_ALIGNMENT);
		content.add(txt3);
		
		JLabel txt4= new JLabel(" ",JLabel.CENTER);
		txt4.setAlignmentX(Component.CENTER_ALIGNMENT);
		content.add(txt4);
		
		setVisible(true);
	}
	
	public void screen1(String error_message){
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		
		JLabel img=Image_2_JLabel(blinds_length);
		img.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridwidth = 3;
		c.gridy=0;
		content.add(img,c);
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=1;
		c.gridwidth = 3;
		content.add(e_txt,c);
		
		JLabel txt2= new JLabel("What is the length of your blinds?",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(txt2,c);
		
		field1 = new JFormattedTextField(NumberFormat.getIntegerInstance());
		field1.setColumns(4);
		field1.addActionListener(new ActionListener(){
			public void actionPerformed (ActionEvent e){
				next();
			}
		});
		c.gridx=2;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(field1,c);
		
		String[] units={"mm","cm","m","inches","feet"};
		list=new JComboBox(units);
		list.setSelectedIndex(0);
		c.gridx=3;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(list,c);
		
		if(b_length!=0)
		{
			field1.setText(Integer.toString(b_length));
			list.setSelectedIndex(0);
		}
		
		field1.requestFocus();
		setVisible(true);
	}
	
	public void screen2(String error_message){
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 2;
		c.insets.left = 2;
		c.insets.top = 10;
		c.insets.bottom = 10;
		
		JLabel img=Image_2_JLabel(blinds_diameter);
		img.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridwidth = 3;
		c.gridy=0;
		content.add(img,c);
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=1;
		c.gridwidth= 3;
		content.add(e_txt,c);
		
		JLabel txt2= new JLabel("Take a look at your blinds top cylinder. What is its diameter?",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(txt2,c);
		
		
		field1 = new JFormattedTextField(NumberFormat.getIntegerInstance());
		field1.setColumns(4);
		field1.addActionListener(new ActionListener(){
			public void actionPerformed (ActionEvent e){
				next();
			}
		});
		field1.setMinimumSize(new Dimension(50,field1.getHeight()));

		c.gridx=2;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(field1,c);
		
		String[] units={"mm","cm","m","inches","feet"};
		list=new JComboBox(units);
		list.setSelectedIndex(0);
		c.gridx=3;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(list,c);
		
		if(b_rad!=0)
		{
			field1.setText(Integer.toString(b_rad));
			list.setSelectedIndex(0);
		}
		
		field1.requestFocus();
		setVisible(true);
	}
	
	public void screen3(String error_message){
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;

		JLabel img1=Image_2_JLabel(blinds_orient_out1);
		img1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=0;
		content.add(img1,c);
		
		but1=new JRadioButton("Image 1");
		c.insets.left = 75;
		c.gridx=1;
		c.gridy=1;
		c.anchor = GridBagConstraints.LINE_END;
		content.add(but1,c);
		
		JLabel img2=Image_2_JLabel(blinds_orient_out2);
		img2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=1;
		c.gridy=2;
		content.add(img2,c);
		
		but2=new JRadioButton("Image 2");
		c.insets.left = 75;
		c.gridx=1;
		c.gridy=3;
		content.add(but2,c);
		
		JLabel img3=Image_2_JLabel(blinds_orient_in1);
		img3.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=2;
		c.gridy=0;
		content.add(img3,c);
		
		but3=new JRadioButton("Image 3");
		c.insets.left = 75;
		c.gridx=2;
		c.gridy=1;
		content.add(but3,c);
		
		JLabel img4=Image_2_JLabel(blinds_orient_in2);
		img4.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=2;
		c.gridy=2;
		content.add(img4,c);
		
		but4=new JRadioButton("Image 4");
		c.insets.left = 75;
		c.gridx=2;
		c.gridy=3;
		content.add(but4,c);
		
		ButtonGroup group=new ButtonGroup();
		group.add(but1);
		group.add(but2);
		group.add(but3);
		group.add(but4);
		
		if(img_selected>0){
			switch(img_selected)
			{
			case 1:
				but1.setSelected(true);
				break;
			case 2:
				but2.setSelected(true);
				break;
			case 3:
				but3.setSelected(true);
				break;
			case 4:
				but4.setSelected(true);
				break;
			}
		}
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=1;
		c.gridy=4;
		c.gridwidth = 2;
		content.add(e_txt,c);
		
		JLabel txt2= new JLabel("Which image resembles your blind installation?",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=5;
		c.gridwidth = 2;
		content.add(txt2,c);
		
		but1.requestFocus();
		setVisible(true);
	}
	
	public void screen4(String error_message){
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;

		JLabel img1=Image_2_JLabel(no_small_internet);
		img1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=0;
		content.add(img1,c);
		
		but1=new JRadioButton("Offline");
		c.insets.left = 75;
		c.gridx=1;
		c.gridy=1;
		c.anchor = GridBagConstraints.LINE_END;
		content.add(but1,c);
		
		JLabel img2=Image_2_JLabel(small_internet);
		img2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=2;
		c.gridy=0;
		content.add(img2,c);
		
		but2=new JRadioButton("Connected");
		c.insets.left = 75;
		c.gridx=2;
		c.gridy=1;
		content.add(but2,c);
		
		ButtonGroup group=new ButtonGroup();
		group.add(but1);
		group.add(but2);
		
		if(connected>0){
			but2.setSelected(true);
		}else{
			but1.setSelected(true);
		}
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=1;
		c.gridy=2;
		c.gridwidth = 2;
		content.add(e_txt,c);
		
		JLabel txt2= new JLabel("Would you like your blinds to be run on offline mode or connected to the internet?",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=3;
		c.gridwidth = 2;
		content.add(txt2,c);
		
		but2.requestFocus();
		setVisible(true);
	}
	
	public void screen5(String error_message){
		//wipe screen and prepare the navigation buttons
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;

		JLabel img1=Image_2_JLabel(internet);
		img1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridwidth = 2;
		c.gridx=1;
		c.gridy=0;
		content.add(img1,c);
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 10;
		c.gridx=1;
		c.gridy=1;
		c.gridwidth = 2;
		content.add(e_txt,c);
		
		JLabel txt2= new JLabel("What is your wireless network name?",JLabel.LEFT);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(txt2,c);
		
		field1 = new JFormattedTextField();
		field1.setColumns(64);
		c.gridx=2;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(field1,c);
		
		if(network!="")
		{
			field1.setText(network);
		}
		
		JLabel txt3= new JLabel("What is your wireless network password?",JLabel.LEFT);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=3;
		c.gridwidth = 1;
		content.add(txt3,c);
		
		field2 = new JPasswordField();
		field2.setColumns(64);
		c.gridx=2;
		c.gridy=3;
		c.gridwidth = 1;
		content.add(field2,c);
		
		if(pwd!="")
		{
			field2.setText(network);
		}
		
		
		setVisible(true);
	}
	
	public void screen6(String error_message){
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 2;
		c.insets.left = 2;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;
		
		JLabel e_txt= new JLabel(error_message,JLabel.CENTER);
		e_txt.setForeground(Color.red);
		e_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=0;
		c.gridy=0;
		c.gridwidth = 8;
		content.add(e_txt,c);
		
		JLabel instruction= new JLabel("If you would like to enter a schedule, add it here. Press next to move on",JLabel.CENTER);
		c.gridx=0;
		c.gridy=1;
		c.gridwidth = 8;
		content.add(instruction,c);
		
		JLabel txt1=new JLabel("ADD");
		txt1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=0;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(txt1,c);
		
		String days[]={"all","Sunday     ","Monday     ","Tuesday","Wednesday","Thursday","Friday       ","Saturday"};
		day_box=new JComboBox(days);
		day_box.setSelectedIndex(0);
		
		c.gridx=1;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(day_box,c);
		
		hour = new JTextField();	
		hour.addActionListener(new ActionListener(){
			public void actionPerformed (ActionEvent e){
				add_scheduled_item(day_box,hour,min,am_pm_box,up_down_box,day_panel);
			}
		});
		hour.setColumns(4);
		c.gridx=2;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(hour,c);
		
		JLabel txt2=new JLabel(":");
		txt1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=3;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(txt2,c);
		
		min = new JTextField();
		min.addActionListener(new ActionListener(){
			public void actionPerformed (ActionEvent e){
				add_scheduled_item(day_box,hour,min,am_pm_box,up_down_box,day_panel);
			}
		});
		min.setColumns(4);
		c.gridx=4;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(min,c);
		
		String am_pm[]={"am","pm"};
		am_pm_box=new JComboBox(am_pm);
		am_pm_box.setSelectedIndex(0);
		
		c.gridx=5;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(am_pm_box,c);
		
		String up_down[]={"up","down"};
		up_down_box=new JComboBox(up_down);
		
		c.gridx=6;
		c.gridy=2;
		c.gridwidth = 1;
		content.add(up_down_box,c);
		
		over_write_box=new JCheckBox();
		if (srv_schd_overwritten==1){
			over_write_box.setSelected(true);
		}else{
			over_write_box.setSelected(false);
		}
		over_write_box.addActionListener(new ActionListener(){
			public void actionPerformed (ActionEvent e){
				handle_overwrite();
			}
		});
		c.insets.left = 75;
		c.gridx=6;
		c.gridy=3;
		c.gridwidth = 1;
		content.add(over_write_box,c);
		
		JLabel txt3=new JLabel("Overwrite Current Server Schedule?");
		txt1.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.insets.left = 2;
		c.gridx=7;
		c.gridy=3;
		c.gridwidth = 1;
		content.add(txt3,c);
		
		JPanel sched=new JPanel();
		sched.setBackground(Color.black);
		sched.setPreferredSize(new Dimension(775,350));
		c.gridx=0;
		c.gridy=4;
		c.gridwidth = 8;
		c.insets.right = 0;
		c.insets.left = 0;
		c.insets.top = 0;
		c.insets.bottom = 0;
		content.add(sched,c);
		
		c.gridy=0;
		c.gridwidth = 1;
		
		for(int i=0;i<7;i++){
			
			c.weighty=99;
			c.gridx=i;
			c.gridy=2;
			day_panel[i]=new JPanel();
			day_panel[i].setLayout(new FlowLayout());
			day_panel[i].setPreferredSize(new Dimension(105,440));
			if(i%2==0){
				day_panel[i].setBackground(new Color(229,229,229));
			}else{
				day_panel[i].setBackground(new Color(233,233,233));
			}
			
			c.gridx=i;
			c.gridy=1;
			c.weighty=1;
			JLabel temp_txt=new JLabel(days[1+i]);
			temp_txt.setAlignmentX(Component.CENTER_ALIGNMENT);
			day_panel[i].add(temp_txt);
			
			sched.add(day_panel[i],c);
		}
		c.weighty=0;
		fill_schedule(day_panel);
		
		JButton submit=new JButton("Submit");
		submit.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					add_scheduled_item(day_box,hour,min,am_pm_box,up_down_box,day_panel);
				}
		});
		c.gridx=7;
		c.gridy=2;
		c.gridwidth = 1;
		c.insets.right = 2;
		c.insets.left = 2;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;
		content.add(submit,c);
		
		setVisible(true);
	}
	
	public void handle_overwrite(){
		if (over_write_box.isSelected()){
			srv_schd_overwritten=1;
		}
		else
		{
			//if we have turned this off, make sure that our schedule array
			//is also blank
			srv_schd_overwritten=0;
			Arrays.fill(sched_array,0);
		}
		//redraw screen 6
		screen6("");
	}
	
	public void add_scheduled_item (JComboBox day_box, 
			JTextField hour, 
			JTextField min, 
			JComboBox am_pm_box,
			JComboBox up_down_box,
			JPanel[] day_panel
			){
		
		int bit_out=0;//value to be written to the schedule array
		int temp=0;
		
		if (day_box.getSelectedItem()=="all"){
			for (int j=0;j<7;j++){
				
				//consider action
				if(up_down_box.getSelectedItem()=="up"){
					bit_out=49152;//0xc000
				}else if(up_down_box.getSelectedItem()=="down"){
					bit_out=32768;//0x8000
				}else{
					screen6("updown error");
					return;
				}
				
				//consider day
				bit_out+=1440*j;//0xc000
				
				//determine time
				try{
					temp=Integer.parseInt(hour.getText());
					
					if (temp==12){
						if (am_pm_box.getSelectedItem()=="pm"){
							bit_out+=12*60;
						}
					}else if(temp<1||temp>12){
						screen6("Please enter valid time");
						return;
					}
					else if(am_pm_box.getSelectedItem()=="am"){
						bit_out+=temp*60;
					}else if(am_pm_box.getSelectedItem()=="pm"){
						bit_out+=temp*60+12*60;
					}else{
						screen6("Error with Hour field");
						return;
					}
				
				}catch (NumberFormatException e){
					screen6("Please format time as a number");
					return;
				}
				
				try{
					temp=Integer.parseInt(min.getText());
					bit_out+=temp;
				
				}catch (NumberFormatException e){
					screen6("Please format time as a number");
					return;
				}
				
				try{
					add_to_sched_array(bit_out);
					srv_schd_overwritten=1;
				}catch (sched_full e){
					screen6(e.getMessage());
				}
			}
			screen6("");
			
		}
		else
		{
			
			if(up_down_box.getSelectedItem()=="up"){
				bit_out=49152;//0xc000
			}else if(up_down_box.getSelectedItem()=="down"){
				bit_out=32768;//0x8000
			}else{
				screen6("updown error");
				return;
			}
			
			int temp2=day_box.getSelectedIndex();
			
			bit_out+=(temp2-1)*1440;
			
			try{
				temp=Integer.parseInt(hour.getText());
				
				if (temp==12){
					if (am_pm_box.getSelectedItem()=="pm"){
						bit_out+=12*60;
					}
				}else if(temp<1||temp>12){
					screen6("Please enter valid time");
					return;
				}
				else if(am_pm_box.getSelectedItem()=="am"){
					bit_out+=temp*60;
				}else if(am_pm_box.getSelectedItem()=="pm"){
					bit_out+=temp*60+12*60;
				}else{
					screen6("Error with Hour field");
					return;
				}
			
			}catch (NumberFormatException e){
				screen6("Please format time as a number");
				return;
			}
			
			try{
				temp=Integer.parseInt(min.getText());
				bit_out+=temp;
			
			}catch (NumberFormatException e){
				screen6("Please format time as a number");
				return;
			}
			
			try{
				add_to_sched_array(bit_out);
				screen6("");
			}catch (sched_full e){
				screen6(e.getMessage());
			}
		}
	}
	
	public void add_to_sched_array (int bit_out) throws sched_full{
		int i=0;
		int temp1,temp2;
		
		if (sched_array[31]!=0){
			throw new sched_full("no more space for events");
		}
		for(i=0;i<31;i++){
			if(((sched_array[i]&0x3fff)==(bit_out&0x3fff))&&((sched_array[i]&0xc000)!=0)){
				throw new sched_full("that time is already being used");
			}
			else if((sched_array[i]&16383)>(bit_out&16383)||sched_array[i]==0){
				temp1=bit_out;
				temp2=sched_array[i];
				for(i=i+1;i<31;i++){
					sched_array[i-1]=temp1;
					temp1=temp2;
					temp2=sched_array[i];
				}
				sched_array[31]=temp1;
				return;
			}
		}
		//if we get here, only the last index is open. Put the next scheduled event here
		sched_array[31]=bit_out;
	}
	
	public void fill_schedule(JPanel[] day_panel){
		String updown="";
		String time="";
		int day;
		int i=0;
		JLabel txt;
		JButton but;
		int c_height[]=new int[7];
		
		GridBagConstraints c = new GridBagConstraints();
		c.gridwidth = 1;
		Arrays.fill(c_height, 1);
		
		while(sched_array[i]!=0&&i<32)
		{
			//get variables
			updown=get_updown(sched_array[i]);
			time=get_time(sched_array[i]);
			day=get_day(sched_array[i]);
			
			//prepare column entry
			txt=new JLabel(time+" "+updown);
			txt.setFont (txt.getFont ().deriveFont (10f));
			c.gridx=0;
			//c.gridy=c_height[day];
			c.gridy=1;
			c.weighty=1;
			day_panel[day].add(txt);
			
			but=new JButton("X");
			but.setActionCommand(Integer.toString(sched_array[i]));
			but.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent e){
					remove_sched(Integer.parseInt(e.getActionCommand()));
				}
			});
			but.setPreferredSize(new Dimension(20, 15));
			but.setMargin(new Insets(0,0,0,0));
			c.gridx=1;
			c.gridy=c_height[day];
			day_panel[day].add(but);
			setVisible(true);
			
			i++;
		}
	}
	
	public String get_time(int bit_stamp){
		String out_str="";
		int time=0;
		int hour;
		int min;
		String am_pm="";

		//prepare a local timestamp (minutes since midnight sunday)
		local_timestamp=0;
		
		time=bit_stamp&16383;
		
		//get only the time data
		time=time%1440;
		min=time%60;
		hour=(int)Math.floor(time/60);
		
		if(time<60*12){
			am_pm="am";
		}else{
			am_pm="pm";
			hour=hour-12;
		}
		
		
		if(hour==0){
			hour=12;
		}
		
		if (min<10){
			out_str=Integer.toString(hour)+":0"+Integer.toString(min)+am_pm;
		}else{
			out_str=Integer.toString(hour)+":"+Integer.toString(min)+am_pm;

		}
		return out_str;
	}
	
	public String get_time_utc(int bit_stamp){
		String out_str="";
		int time=0;
		int hour;
		int min;
		String am_pm="";
		
		TimeZone z;
		int utc_factor;
		Calendar cal=Calendar.getInstance();

		//prepare a local timestamp (minutes since midnight sunday)
		local_timestamp=0;
		
		//determine factor necessary to convert system time back to UTC
		z=cal.getTimeZone();
		utc_factor=z.getRawOffset();
		if(z.inDaylightTime(new Date())){
			utc_factor=utc_factor+z.getDSTSavings();
		}
		utc_factor=-(utc_factor/1000/60);
		
		
		time=bit_stamp&16383;
		time=time-utc_factor;
		
		//get only the time data
		time=time%1440;
		min=time%60;
		hour=(int)Math.floor(time/60);
		
		if(time<60*12){
			am_pm="am";
		}else{
			am_pm="pm";
			hour=hour-12;
		}
		
		
		if(hour==0){
			hour=12;
		}
		
		if (min<10){
			out_str=Integer.toString(hour)+":0"+Integer.toString(min)+am_pm;
		}else{
			out_str=Integer.toString(hour)+":"+Integer.toString(min)+am_pm;

		}
		return out_str;
	}
	
	public void remove_sched(int bit_stamp){
		int i;
		int j;
		
		for(i=0;i<31;i++){
			if(sched_array[i]==bit_stamp){
				for (j=i;j<31;j++){
					sched_array[j]=sched_array[j+1];
				}
				break;
			}
		}
		
		if(i==30&&sched_array[31]==bit_stamp){
			sched_array[31]=0;
		}
		
		screen6("");
	}

	public int get_day(int bit_stamp){
		int time=0;
		time=bit_stamp&16383;
		
		return(int)Math.floor(time/1440);
	}
	
	public String get_updown(int mem_entry){
		//assume we are being fed integers formatted like 
		//regular memory
		String out_str="";
		if ((mem_entry&49152)==49152){
			out_str="up";
		}else if ((mem_entry&32768)==32768){
			out_str="down";
		}else{
			out_str="err";
		}
		
		return out_str;
	}
	
	public void screen7(String error_message){
		JPanel content=prepare_template();
		
		content.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill=GridBagConstraints.BOTH;
		c.insets.right = 10;
		c.insets.left = 10;
		c.insets.top = 10;
		c.insets.bottom = 10;
		c.gridwidth = 1;
		
		JLabel img=Image_2_JLabel(icon);
		img.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=1;
		content.add(img,c);
		
		JLabel txt2= new JLabel("Your SD card is ready to be inserted into your blinds!",JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=2;
		content.add(txt2,c);
		
		txt2= new JLabel("Please note that your blinds will think that it is "+get_time_utc(local_timestamp),JLabel.CENTER);
		txt2.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.gridx=1;
		c.gridy=3;
		content.add(txt2,c);
		
		setVisible(true);
	}
	
	public static void main(String[] args) throws IOException {
		EventQueue.invokeLater(new Runnable() {
			@Override
			public void run() {
				wizard_swing ex = new wizard_swing();
				ex.setVisible(true);
			}
		});
	}
}
