import socket
import time
import select
import sys
import traceback
import struct
import calendar
import math
import datetime
import MySQLdb

#code operation variables
comments=1;

#networking variables
PORT=9993
period=['\x00\x64','\x00\x03\xe8','\x00\x07\xd0']
index=0
HOST='130.113.54.158'

#ySQL Variables
SQL_HOST='localhost'
SQL_USER='blinds_handler'
SQL_PWD='raspberry1python'
SQL_DB='shade_soleil'

#communication variables (treat them as constants)
SER_INDEX_1=0
SER_INDEX_2=1
REQ_INDEX=2
BLIND_HEIGHT_INDEX=3
CT_INDEX_1=4
CT_INDEX_2=5
SCHEDULE_INDEX=6

class packet_error(Exception):
    def __init__(self,value):
        self.value=value
    def __Str__(self,value):
        return repr(self,value)

    
def process_packet (raw):
    #serial number
    sn=raw[SER_INDEX_1]*256+raw[SER_INDEX_2]
    #ntype of message inquiry
    msg_type=raw[REQ_INDEX]
    #what is the heigh of the blinds
    ht_prct=raw[BLIND_HEIGHT_INDEX]
    #what is the blind's local timestamp
    loc_ts=raw[CT_INDEX_1]*256+raw[CT_INDEX_2]
    #when were the blinds last updated
    unix_ts=raw[SCHEDULE_INDEX]*2**56
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+1]*2**48
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+2]*2**40
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+3]*2**32
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+4]*2**24
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+5]*2**16
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+6]*2**8
    unix_ts=unix_ts+raw[SCHEDULE_INDEX+7]
    #pass on the raw packet as well just in case it needs further processing

    return {'sn':sn,'msg_type':msg_type,'ht_prct':ht_prct,'loc_ts':loc_ts,'unix_ts':unix_ts,'raw':raw}

def get_local_ts():
    d=datetime.datetime.today().weekday()
    d=d+1
    if d==7:d=0
    lt=datetime.datetime.today().minute+datetime.datetime.today().hour*60+d*24*60
    return lt

def mysql_query(query):
    
    db=MySQLdb.connect(SQL_HOST,SQL_USER,SQL_PWD,SQL_DB)
    c=db.cursor()
    c.execute(query)
    db.commit()
    ans=c.fetchall()
    db.close()
    return ans
        
def get_php_srv_checkin(b_id):
    query='SELECT last_check_in from blinds where b_id='+str(b_id)+';'
    resp=mysql_query(query)
    return resp[0][0]

def prepare_packet_head(msg_type, pp):
    lt=get_local_ts()
    
    resp=[0]*6
    resp[SER_INDEX_1]=(pp['sn']>>8)&0xff
    resp[SER_INDEX_2]=pp['sn']&0xff
    resp[REQ_INDEX]=msg_type
    resp[BLIND_HEIGHT_INDEX]=0
    resp[CT_INDEX_1]=(lt>>8)&0xff
    resp[CT_INDEX_2]=lt&0xff

    return resp

def prepare_packet_schedule(b_id):
    #this assumes that the maximum number of events (32) is not exceeded
    schd=mysql_query('SELECT * FROM b_'+str(b_id))

    t=mysql_query('SELECT svr_check_in FROM blinds WHERE b_id='+str(b_id))
    t=t[0][0]
    
    max_index=32
    if len(schd)<max_index:
        max_index=len(schd)

    resp=[0]*128

    #insert unix timestamp
    resp[0]=(t>>56)&0xff
    resp[1]=(t>>48)&0xff
    resp[2]=(t>>40)&0xff
    resp[3]=(t>>32)&0xff
    resp[4]=(t>>24)&0xff
    resp[5]=(t>>16)&0xff
    resp[6]=(t>>8)&0xff
    resp[7]=(t)&0xff

    #note that the 4 next bytes are reserved (indices 8-11)
    #to elaborate, these 4 bytes are used to hold the repetitive event
    #the control circuit uses to remember when to next ping the server.
    #whatever we put in them will be overwritten

    for i in range (0,max_index):
        
        #code appropriate event type
        if(schd[i][1]==2):
            #down event is indicated with a 0x80
            resp[64+2*i]=0x80
        else:
            #up is 0xc0
            resp[64+2*i]=0xC0
        #time is stored in seconds since sunday (?!? why did i do that?)
        #perhaps it allows me to switch to more accurate time keeping in the future
        event_time=(schd[i][0])/60
        
        #code appropriate time
        resp[64+2*i]=resp[64+2*i]+((event_time>>8)&0xff)
        resp[64+2*i+1]=(event_time&0xff)

    return resp

def parse_schedule(pp):
    # clear the table and make room for more data
    query='DELETE FROM b_'+str(pp['sn'])
    mysql_query(query)
    
    for i in range(64,128,2):
        if(pp['raw'][i])&0xff:
            if (pp['raw'][i])&0xc0==0xC0:
                #this is an up event
                updown=1
            elif (pp['raw'][i])&0xc0==0x80:
                #this is a down event
                updown=0
            else:
                #this is a weird xbee event that should be ignored or a blank item
                updown=-1

            if updown!=-1:
                #if we have a valid event, continue to parse
                event_time=((pp['raw'][i])&0x3f)*256
                event_time=event_time+((pp['raw'][i+1])&0xff)
                #convert the event time into seconds (they're stored in seconds on mysql)
                event_time=event_time*60

                query='INSERT INTO b_'+str(pp['sn'])+' (time, updown) VALUES ('+str(event_time)+','+str(updown)+')'
                mysql_query(query)

    #end this by updating the server updated field (this indicates the version of the schedule held in mysql
    query='UPDATE  blinds SET svr_check_in='+str(pp['unix_ts'])+' WHERE b_id='+str(pp['sn'])
    mysql_query(query)
    
def mysql_newer(pp):
    #this function compares the unix timestamps of the parsed packet to the one stored in the mysql server
    #it returns true if mysql is newer
    query='SELECT svr_check_in FROM blinds WHERE b_id='+str(pp['sn'])
    php_unix_ts=mysql_query(query)
    if(php_unix_ts[0][0]>pp['unix_ts']):
        result=1
    else:
        result=0

    return result

def prep_resp_schd_changes(pp):
    #determine input, consult MySQL to determine correct response packet
    #pp stands for parsed packet
    lt=get_local_ts()
    resp=prepare_packet_head(1,pp)

    if (pp['msg_type']==2):
        ##bootup

        if mysql_newer(pp):
            #we must respond with an updated schedule
            resp=prepare_packet_head(4,pp)+prepare_packet_schedule(pp['sn'])
        else:
            #request the blind's current schedule to keep the server up to date
            resp=prepare_packet_head(3,pp)
            
    elif (pp['msg_type']==3):
        #client sending schedule we must parse it and update the schedule in mysql

        #double check that the new packet is in fact newer than the currently loaded schedule
        if(not mysql_newer(pp)):
            parse_schedule(pp)

        #return a generic packet so that the circuit can maintain an accurate clock
        resp=resp
        
    elif (pp['msg_type']==4):
        #checkin
        if mysql_newer(pp):
            #we must respond with an updated schedule
            resp=prepare_packet_head(4,pp)+prepare_packet_schedule(pp['sn'])
        else:
            #send an empty packet (msg_type!=3,4), the blinds can use this to
            #double check their local timestamp, and stop sending messages
            resp=resp
    else:
        #unknown input, return a -1
        raise packet_error('unknown message type: ',pp['msg_type'])

    return resp

def update_mysql_server(pp):

    t=calendar.timegm(time.gmtime())
    #determin if a table exists for this blind
    exists=mysql_query('show tables like \'b_'+str(pp['sn'])+'\'')
    if (not exists):
        #if it does not exist, make a new table based off of the b_0 table
        mysql_query('CREATE TABLE b_'+str(pp['sn'])+' LIKE b_0')
        #insert info into the blinds table about last checkins
        mysql_query('INSERT INTO blinds (p_id,last_check_in,state,b_id,svr_check_in,blind_height) VALUES(-1,0,\'down\','+str(pp['sn'])+',0,0)') 

    #now a table exists for these blinds, update the relevant info
    mysql_query('UPDATE blinds SET last_check_in='+str(t)+' WHERE b_id='+str(pp['sn']))
    mysql_query('UPDATE blinds SET blind_height='+str(pp['ht_prct'])+' WHERE b_id='+str(pp['sn']))

def fake_packet():
    p=[0]*14
    i=0
    p[i]=0x0
    i=i+1
    p[i]=0x1
    i=i+1
    p[i]=0x04
    i=i+1
    p[i]=75
    i=i+1
    p[i]=0x09
    i=i+1
    p[i]=0xc4
    i=i+1
    p[i]=0x0
    i=i+1
    p[i]=0x0
    i=i+1
    p[i]=0x0
    i=i+1
    p[i]=0x0
    i=i+1
    p[i]=0x54
    i=i+1
    p[i]=0xb9
    i=i+1
    p[i]=0x3d
    i=i+1
    p[i]=0x44
    i=i+1


    return p

def main():
    if comments: print HOST
    if comments: print PORT
    if comments: print 'Program Start'

    while 1:

        #reset or start variables
        s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
        s.bind((HOST,PORT))
        s.listen(5)

        try:
            if comments: print 'listening'
            conn,addr=s.accept()
        except:
            if comments: print 'error listening:', sys.exc_info()[0]
            if comments: sys.exc_info()[2]

        try:
            s=conn.recv(200)
            raw=[ord(c) for c in s]
            if comments: print 'Received: ',raw
        except:
            if comments: print 'failed recv', sys.exc_info()[0]
            conn.close()
            
        try:
            processed_packet=process_packet(raw)

            if comments: print 'msg_type=',processed_packet['msg_type']
            if comments and processed_packet['msg_type']==2: print 'bootup'
            if comments and processed_packet['msg_type']==3: print 'CLT_SEND'
            if comments and processed_packet['msg_type']==4: print 'SRV_SEND (CHECKIN)'
            if comments: print 'ht_prct=',processed_packet['ht_prct']
            if comments: print 'loc_ts=',processed_packet['loc_ts']
            if comments: print 'unix_ts=',processed_packet['unix_ts']

        except:
            if comments:print 'error parsing packet:', sys.exc_info()[0]

        try:
            update_mysql_server(processed_packet)
            if comments:print 'updated mysql server'

        except:
            if comments:print 'error updating mysql server:', sys.exc_info()[0]
            
        try:
            resp= prep_resp_schd_changes(processed_packet)
            if comments: print 'prepared response: ',resp

        except:
            if comments: print 'error preparing response: ', sys.exc_info()[0]
            if comments: traceback.print_tb(sys.exc_info()[2])

        try:
            conn.send(period[index])
            conn.send(''.join([chr(c)for c in resp]))
            if comments: print 'sent response'

        except:
                if comments:print 'error sending response: ', sys.exc_info()[0]

        conn.close()
        

        

        
